@@ -8,9 +8,7 @@
 #define BUILTIN_H
 
 #include "vm.h"
-namespace types {
-class ty;
-}
+#include "types.h"
 
 namespace trans {
 
@@ -23,11 +21,15 @@ void base_tenv(tenv &);
 void base_venv(venv &);
 void base_menv(menv &);
 
+extern const types::formal noformal;
+  
+// Add a function with one or more default arguments.
 void addFunc(venv &ve, vm::bltin f, types::ty *result, const char *name, 
-             types::ty *t1 = 0, types::ty *t2 = 0, types::ty *t3 = 0,
-             types::ty *t4 = 0, types::ty *t5 = 0, types::ty *t6 = 0,
-             types::ty *t7 = 0, types::ty *t8 = 0);
-
+	     types::formal f1=noformal, types::formal f2=noformal,
+	     types::formal f3=noformal, types::formal f4=noformal,
+	     types::formal f5=noformal, types::formal f6=noformal,
+	     types::formal f7=noformal, types::formal f8=noformal);
+  
 } //namespace trans
 
 #endif //BUILTIN_H
