@@ -21,8 +21,7 @@ namespace absyntax {
 
 using namespace trans;
 using namespace types;
-
-
+  
 trans::tyEntry *ty::transAsTyEntry(coenv &e)
 {
   return new trans::tyEntry(trans(e, false), 0);
@@ -57,64 +56,39 @@ void arrayTy::prettyprint(ostream &out, int indent)
 
 function *arrayTy::opType(types::ty* t)
 {
-  function *ft = new function(primBoolean());
-  ft->add(t);
-  ft->add(t);
-
-  return ft;
+  return new function(primBoolean(),types::formal(t,"a"),types::formal(t,"b"));
 }
 
 function *arrayTy::arrayType(types::ty* t)
 {
-  function *ft = new function(t);
-  ft->add(t);
-
-  return ft;
+  return new function(t,types::formal(t,"a"));
 }
 
 function *arrayTy::array2Type(types::ty* t)
 {
-  function *ft = new function(t);
-  ft->add(t);
-  ft->add(t);
-
-  return ft;
+  return new function(t,types::formal(t,"a"),types::formal(t,"b"));
 }
 
 function *arrayTy::cellIntType(types::ty* t)
 {
-  function *ft = new function(t);
-  ft->add(primInt());
-  
-  return ft;
+  return new function(t,primInt());
 }
   
 function *arrayTy::sequenceType(types::ty* t, types::ty *ct)
 {
-  function *ft = new function(t);
-  function *fc = cellIntType(ct);
-  ft->add(fc);
-  ft->add(primInt());
-
-  return ft;
+  return new function(t,types::formal(cellIntType(ct),"f"),
+		      types::formal(primInt(),"n"));
 }
 
 function *arrayTy::cellTypeType(types::ty* t)
 {
-  function *ft = new function(t);
-  ft->add(t);
-  
-  return ft;
+  return new function(t,t);
 }
   
 function *arrayTy::mapType(types::ty* t, types::ty *ct)
 {
-  function *ft = new function(t);
-  function *fc = cellTypeType(ct);
-  ft->add(fc);
-  ft->add(t);
-
-  return ft;
+  return new function(t,types::formal(cellTypeType(ct),"f"),
+		      types::formal(t,"a"));
 }
 
 void arrayTy::addOps(coenv &e, types::ty* t, types::ty *ct)
@@ -201,15 +175,12 @@ void block::transAsField(coenv &e, record *r)
 void block::transAsRecordBody(coenv &e, record *r)
 {
   transAsField(e, r);
-
-  // Put record into finished state.
-  e.c.encode(inst::pushclosure);
-  e.c.close();
+  e.c.closeRecord();
 }
 
 void block::transAsFile(coenv &e, record *r)
 {
-  if (settings::autoplain) {
+  if (settings::getSetting<bool>("autoplain")) {
     autoplainRunnable()->transAsField(e, r);
   }
 
@@ -514,10 +485,7 @@ class loadModuleExp : public exp {
 
 public:
   loadModuleExp(position pos, record *imp)
-    : exp(pos), imp(imp), ft(new function(imp))
-  {
-    ft->add(primString());
-  }
+    : exp(pos), imp(imp), ft(new function(imp,primString())) {}
 
   types::ty *trans(coenv &) {
     em->compiler(getPos());
@@ -659,7 +627,8 @@ void unraveldec::prettyprint(ostream &out, int indent)
 {
   prettyname(out, "unraveldec", indent);
   id->prettyprint(out, indent+1);
-  this->fields->prettyprint(out, indent+1);
+  idpairlist *f=this->fields;
+  if(f) f->prettyprint(out, indent+1);
 }
 
 varEntry *unraveldec::getQualifier(coenv &e, record *)
@@ -674,7 +643,8 @@ void fromaccessdec::prettyprint(ostream &out, int indent)
 {
   prettyindent(out, indent);
   out << "fromaccessdec '" << *id << "'\n";
-  this->fields->prettyprint(out, indent+1);
+  idpairlist *f=this->fields;
+  if(f) f->prettyprint(out, indent+1);
 }
 
 varEntry *fromaccessdec::getQualifier(coenv &e, record *r)
@@ -703,6 +673,9 @@ void includedec::loadFailed(coenv &)
 
 void includedec::transAsField(coenv &e, record *r)
 {
+  if(settings::verbose > 1)
+    std::cerr << "Including " <<  filename << std::endl;
+  
   file *ast = parser::parseFile(filename);
   em->sync();
 
@@ -729,21 +702,18 @@ void recorddec::prettyprint(ostream &out, int indent)
 
 function *recorddec::opType(record *r)
 {
-  function *ft = new function(primBoolean());
-  ft->add(r);
-  ft->add(r);
-
-  return ft;
+  return new function(primBoolean(),types::formal(r,"a"),
+		      types::formal(r,"b"));
 }
 
-void recorddec::addOps(coenv &e, record *r)
+void recorddec::addOps(coenv &e, record *parent, record *r)
 {
   function *ft = opType(r);
   varEntry *ve=new varEntry(ft, new bltinAccess(run::boolMemEq));
-  e.e.addVar(symbol::trans("alias"), ve);
-  e.e.addVar(symbol::trans("=="), ve);
-  e.e.addVar(symbol::trans("!="),
-      new varEntry(ft, new bltinAccess(run::boolMemNeq)));
+  addVar(e,parent,ve,symbol::trans("alias"));
+  addVar(e,parent,ve,symbol::trans("=="));
+  addVar(e,parent,new varEntry(ft, new bltinAccess(run::boolMemNeq)),
+	 symbol::trans("!="));
 }
 
 void recorddec::transAsField(coenv &e, record *parent)
@@ -756,7 +726,7 @@ void recorddec::transAsField(coenv &e, record *parent)
   if (parent)
     parent->e.addType(id, ent);
   e.e.addType(id, ent);
-  addOps(e,r);
+  addOps(e,parent,r);
 
   // Start translating the initializer.
   coder c=e.c.newRecordInit(r);
