@@ -357,7 +357,7 @@ void newPictureExp::prettyprint(ostream &out, int indent)
 
 types::ty *newPictureExp::trans(coenv &e)
 {
-  e.c.encode(inst::builtin, run::newFrame);
+  e.c.encode(inst::builtin, run::newPicture);
   
   return types::primPicture();  
 }
@@ -421,7 +421,7 @@ void argument::prettyprint(ostream &out, int indent)
 }
 
 void argument::assignAmbiguity(coenv &e) {
-  if (settings::debug && name && e.e.varGetType(name)) {
+  if (name && e.e.varGetType(name) && settings::getSetting<bool>("debug")) {
     em->warning(val->getPos());
     *em << "named argument may be mistaken for assignment";
   }
@@ -585,8 +585,8 @@ types::ty *callExp::trans(coenv &e)
   if (!a)
     return primError();
 
-  // To simulate left-to-right order of evaluation, produce the side-effects for
-  // the callee.
+  // To simulate left-to-right order of evaluation, produce the
+  // side-effects for the callee.
   assert(a);
   function *t=a->getType();
   assert(t);
