@@ -18,11 +18,12 @@ string asyinput=".asy_input";
 ofile nullfile("");
 ofile Stdout("");
 
-void ifile::ignoreComment()
+void ifile::ignoreComment(bool readstring)
 {
   if(comment == 0) return;
   int c;
-  if(stream->peek() == '\n') return;
+  bool eol=(stream->peek() == '\n');
+  if(eol && (readstring || (csvmode && nullfield))) return;
   for(;;) {
     while(isspace(c=stream->peek())) {
       stream->ignore();
@@ -33,11 +34,24 @@ void ifile::ignoreComment()
       while((c=stream->peek()) != '\n' && c != EOF)
 	stream->ignore();
       if(c == '\n') stream->ignore();
-    } else return;
+    } else {if(eol) stream->unget(); return;}
   }
 }
   
 bool ifile::eol()
+{
+  int c;
+  while(isspace(c=stream->peek())) {
+    if(c == '\n') return true;
+    else {
+      stream->ignore();
+      whitespace += (char) c;
+    }
+  }
+  return false;
+}
+  
+bool ifile::nexteol()
 {
   int c;
   if(nullfield) {
@@ -87,7 +101,7 @@ mem::string ifile::getcsvline()
     int c=stream->peek();
     if(c == '"') {quote=!quote; stream->ignore(); continue;}
     if(!quote && (c == ',' || c == '\n')) {
-      if(c == '\n') ignoreComment();
+      if(c == '\n') ignoreComment(true);
       return s;
     }
     s += (char) stream->get();
