@@ -50,7 +50,7 @@ public:
 
   void resetlines() {lines=0;}
   
-  bool Standard() {resetlines(); return standard;}
+  bool Standard() {return standard;}
   
   void dimension(int Nx=-1, int Ny=-1, int Nz=-1) {nx=Nx; ny=Ny; nz=Nz;}
   
@@ -84,6 +84,7 @@ public:
 		
   string filename() {return name;}
   virtual bool eol() {return false;}
+  virtual bool nexteol() {return false;}
   virtual bool text() {return false;}
   virtual bool eof()=0;
   virtual bool error()=0;
@@ -91,6 +92,8 @@ public:
   virtual void clear()=0;
   virtual void precision(int) {}
   virtual void flush() {}
+  virtual size_t tell() {return 0;}
+  virtual void seek(size_t) {}
   
   void unsupported(const char *rw, const char *type) {
     std::ostringstream buf;
@@ -119,7 +122,7 @@ public:
   virtual void write(const triple&) {nowrite("triple");}
   virtual void write(const mem::string&) {nowrite("string");}
   virtual void write(const pen&) {nowrite("pen");}
-  virtual void write(const guide&) {nowrite("guide");}
+  virtual void write(guide *) {nowrite("guide");}
   virtual void write(const transform&) {nowrite("transform");}
   virtual void writeline() {nowrite("string");}
   
@@ -168,12 +171,17 @@ public:
     if(!standard && !closed) fstream.seekg(pos);
   }
   
+  size_t tell() {
+    return fstream.tellg();
+  }
+  
   const char* Mode() {return "input";}
   
   void csv();
   
-  void ignoreComment();
+  void ignoreComment(bool readstring=false);
   bool eol();
+  bool nexteol();
   
   bool text() {return true;}
   bool eof() {return stream->eof();}
@@ -257,7 +265,7 @@ public:
   void write(const triple& val) {*stream << val;}
   void write(const mem::string& val) {*stream << val;}
   void write(const pen& val) {*stream << val;}
-  void write(const guide& val) {*stream << val;}
+  void write(guide *val) {*stream << *val;}
   void write(const transform& val) {*stream << val;}
   void writeline() {
     if(standard && settings::scrollLines) {
@@ -304,7 +312,8 @@ public:
   void read(triple& val) {
     double x=0.0, y=0.0, z=0.0;
     stream >> x >> y >> z;
-    val=triple(x,y,z);}
+    val=triple(x,y,z);
+  }
 };
 
 class oxfile : public file {
@@ -349,7 +358,7 @@ void ifile::iread(T& val)
   if(standard) clear();
   if(errorstream::interrupt) throw interrupted();
   else {
-    ignoreComment();
+    ignoreComment(typeid(T)==typeid(mem::string));
     val=T();
     if(!nullfield)
       Read(val);
