@@ -24,6 +24,10 @@
 using std::cout;
 using namespace settings;
 
+namespace run {
+  void init_readline(bool);
+}
+
 namespace interact {
 
 int interactive=false;
@@ -31,13 +35,25 @@ bool virtualEOF=true;
 bool resetenv;
 bool uptodate=true;
 
+void init_interactive() 
+{
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-
-static string historyname="history";
-static string localhistoryname=".asy_history";
+  run::init_readline(false);
+  read_history(historyname.c_str());
+#endif  
+}
+  
+#if !defined(HAVE_LIBREADLINE) || !defined(HAVE_LIBCURSES)
+char *readline(const char *prompt) {
+  std::cout << prompt;
+  string s;
+  getline(std::cin,s);
+  return strdup(s.c_str());
+}
+#endif  
   
 /* Read a string, and return a pointer to it. Returns NULL on EOF. */
-char *rl_gets(void)
+const char *rl_gets()
 {
   static char *line_read=NULL;
   /* If the buffer has already been allocated,
@@ -56,26 +72,30 @@ char *rl_gets(void)
       if(pid) restart=(waitpid(pid, &status, WNOHANG) == pid);
       if(restart) {
 	ostringstream cmd;
-	cmd << PDFViewer << " " << docdir << "/asymptote.pdf";
-	status=System(cmd,false,false,"ASYMPTOTE_PDFVIEWER","pdf viewer",
-		      &pid);
+	cmd << "'" << getSetting<mem::string>("pdfviewer") << "' " 
+	    << docdir << "/asymptote.pdf";
+	status=System(cmd,false,false,"pdfviewer","your PDF viewer",&pid);
       }
       continue;
     }
     break;
   }
      
-  if(!line_read) cout << endl;
-  else {
+  if(line_read) {
     if(strcmp(line_read,"q") == 0 || strcmp(line_read,"quit") == 0
        || strcmp(line_read,"quit;") == 0
        || strcmp(line_read,"exit") == 0
        || strcmp(line_read,"exit;") == 0)
       return NULL;
-  }
   
-  /* If the line has any text in it, save it on the history. */
-  if(line_read && *line_read) add_history(line_read);
+    /* If the line has any text in it, save it on the history. */
+#if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
+    if(*line_read) add_history(line_read);
+#endif    
+  } else {
+    cout << endl;
+    return "\n";
+  }
   
   return line_read;
 }
@@ -108,26 +128,15 @@ static size_t ninputexpand=strlen(inputexpand);
   
 size_t interactive_input(char *buf, size_t max_size)
 {
-  static int nlines=1000;
-  static bool first=true;
-  static string historyfile;
   static bool inputmode=false;
     
   assert(max_size > 0);
   size_t size=max_size-1;
   char *to=buf;
     
-  if(first) {
-    first=false;
-    historyfile=localhistory ? localhistoryname : (initdir+historyname);
-    
-    read_history(historyfile.c_str());
-    rl_bind_key('\t',rl_insert); // Turn off tab completion
-  }
-
   if(virtualEOF) return 0;
   
-  static char *line;
+  static const char *line;
 
   if(inputmode) {
     inputmode=false;  
@@ -162,12 +171,12 @@ size_t interactive_input(char *buf, size_t max_size)
     add_input(to,line,size);
     return to-buf;
   } else {
-    stifle_history(nlines);
-    write_history(historyfile.c_str());
+#if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
+    stifle_history(getSetting<int>("historylines"));
+    write_history(historyname.c_str());
+#endif    
     return 0;
   }
 }
 
-#endif
-
 } // namespace interact
