@@ -8,14 +8,6 @@
 #include "config.h"
 #endif
 
-namespace camp {
-extern bool TeXcontaminated;
-}
-
-namespace run {
-  void cleanup();
-}
-
 namespace interact {
 
 extern int interactive;
@@ -24,6 +16,7 @@ extern bool resetenv;
 extern bool uptodate;
 
 size_t interactive_input(char *buf, size_t max_size);
+void init_interactive();
   
 #define YY_READ_BUF_SIZE YY_BUF_SIZE
   
