@@ -17,6 +17,7 @@
 #include "genv.h"
 #include "stm.h"
 #include "settings.h"
+#include "locate.h"
 #include "vm.h"
 #include "program.h"
 #include "interact.h"
@@ -26,6 +27,12 @@
 #include "stack.h"
 #include "runtime.h"
 
+namespace run {
+  void cleanup();
+  void exitFunction(vm::stack *Stack);
+  void updateFunction(vm::stack *Stack);
+}
+
 using namespace settings;
 using std::list;
 
@@ -45,7 +52,7 @@ using interact::uptodate;
 #ifdef HAVE_LIBSIGSEGV
 void stackoverflow_handler (int, stackoverflow_context_t)
 {
-  em->runtime(vm::getPos());
+  if(em) em->runtime(vm::getPos());
   cout << "Stack overflow" << endl;
   abort();
 }
@@ -53,7 +60,7 @@ void stackoverflow_handler (int, stackoverflow_context_t)
 int sigsegv_handler (void *, int emergency)
 {
   if(!emergency) return 0; // Really a stack overflow
-  em->runtime(vm::getPos());
+  if(em) em->runtime(vm::getPos());
   cout << "Segmentation fault" << endl;
   cout << "Please report this programming error to" << endl 
        << BUGREPORT << endl;
@@ -104,12 +111,6 @@ void purge()
 #endif
 }
 
-void doPrint(genv&, record *m)
-{
-  // NOTE: Should make it possible to show more code.
-  print(cout, m->getInit()->code);
-}
-
 // Run (an already translated) module of the given filename.
 void doRun(genv& ge, std::string filename)
 {
@@ -123,30 +124,11 @@ typedef vm::interactiveStack istack;
 using absyntax::runnable;
 using absyntax::block;
 
-mem::vector<coenv*> estack;
-mem::vector<vm::interactiveStack*> sstack;
-
 // Abstract base class for the core object being run in line-at-a-time mode, it
 // may be a runnable, block, file, or interactive prompt.
 struct icore {
   virtual ~icore() {}
-  
   virtual void run(coenv &e, istack &s) = 0;
-  
-  // Wrapper for run used to execute eval within the current environment.
-  void wrapper(coenv &e, istack &s, bool exit=false) {
-    estack.push_back(&e);
-    sstack.push_back(&s);
-    run(e,s);
-    if(exit) run::exitFunction(&s);
-    estack.pop_back();
-    sstack.pop_back();
-  }
-  
-  void embedded() {
-    assert(estack.size() && sstack.size());
-    run(*(estack.back()),*(sstack.back()));
-  };
 };
 
 struct irunnable : public icore {
@@ -160,6 +142,7 @@ struct irunnable : public icore {
     lambda *codelet=r->transAsCodelet(e);
     em->sync();
     if(!em->errors()) {
+      if(getSetting<bool>("translate")) print(cout,codelet->code);
       s.run(codelet);
     } else {
       e.e.endScope(); // Remove any changes to the environment.
@@ -171,34 +154,46 @@ struct irunnable : public icore {
 struct itree : public icore {
   absyntax::block *ast;
 
+  itree() : ast(0) {}
+  
   itree(absyntax::block *ast)
     : ast(ast) {}
 
+  void set(block *a) {ast=a;}
+  
   void run(coenv &e, istack &s) {
-    for(list<runnable *>::iterator r=ast->stms.begin(); r != ast->stms.end();
-	++r)
-      irunnable(*r).wrapper(e,s);
+    for(list<runnable *>::iterator r=ast->stms.begin();
+	r != ast->stms.end(); ++r)
+      if(!em->errors() || getSetting<bool>("debug")) 
+	irunnable(*r).run(e,s);
   }
 };
 
-struct iprompt : public icore {
+struct iprompt : public itree {
+  iprompt() : itree() {}
+  
+  iprompt(absyntax::block *ast)
+    : itree(ast) {}
+  
   void run(coenv &e, istack &s) {
-    while (virtualEOF && !resetenv) {
-      virtualEOF=false;
-      try {
-        file *ast = parser::parseInteractive();
-        assert(ast);
-        itree(ast).wrapper(e,s);
-	if(!uptodate)
-	  run::updateFunction(&s);
-      } catch (interrupted&) {
-        if(em) em->Interrupt(false);
-        cout << endl;
-      } catch (handled_error) {
-        status=false;
+    virtualEOF=true;
+    while (virtualEOF) {
+      if(!ast) {
+	virtualEOF=false;
+	set(parser::parseInteractive());
+	if(resetenv) {purge(); break;}
       }
+      try {
+	itree::run(e,s);
+      } catch (interrupted&) {
+	if(em) em->Interrupt(false);
+	cout << endl;
+      }
+      ast=0;
+      if(!uptodate && virtualEOF)
+	run::updateFunction(&s);
+      purge(); // Close any files that have gone out of scope.
     }
-    run::cleanup();
   }
 };
 
@@ -208,36 +203,51 @@ void doICore(icore &i, bool embedded=false) {
   if(em->errors()) return;
   
   try {
-    if(embedded)
-      i.embedded();
-    else {
+    static mem::vector<coenv*> estack;
+    static mem::vector<vm::interactiveStack*> sstack;
+    if(embedded) {
+      assert(estack.size() && sstack.size());
+      i.run(*(estack.back()),*(sstack.back()));
+    } else {
       purge();
       
       genv ge;
       env base_env(ge);
       coder base_coder;
       coenv e(base_coder,base_env);
-
+      
       vm::interactiveStack s;
       s.setInitMap(ge.getInitMap());
 
-      if(settings::autoplain) {
+      estack.push_back(&e);
+      sstack.push_back(&s);
+
+      if(settings::getSetting<bool>("autoplain")) {
 	absyntax::runnable *r=absyntax::autoplainRunnable();
-	irunnable(r).wrapper(e,s);
+	irunnable(r).run(e,s);
       }
 
       // Now that everything is set up, run the core.
-      i.wrapper(e,s,true);
-
-      if(settings::listvariables)
+      i.run(e,s);
+      
+      if(interactive) {
+	interactive=false;
+	run::exitFunction(&s);
+	interactive=true;
+      } else run::exitFunction(&s);
+      
+      estack.pop_back();
+      sstack.pop_back();
+      
+      if(settings::getSetting<bool>("listvariables"))
 	base_env.list();
-    
     }
   } catch (std::bad_alloc&) {
     cerr << "error: out of memory" << endl;
     status=false;
   } catch(handled_error) {
     status=false;
+    run::cleanup();
   }
 
   em->clear();
@@ -259,41 +269,65 @@ void doIFile(const string& filename) {
   init();
 
   string basename = stripext(filename,suffix);
-  if(verbose) cout << "Processing " << basename << endl;
+  if(settings::verbose) cout << "Processing " << basename << endl;
   
-  if(parseonly) {
-    absyntax::file *tree = parser::parseFile(filename);
-    em->sync();
-    if(!em->errors())
-      tree->prettyprint(cout, 0);
-    else status=false;
-  } else if(translate) {
-    genv ge;
-    record *m = ge.getModule(symbol::trans(basename),filename);
-    if(!em->errors())
-      doPrint(ge,m);
-  } else {
-    if(filename == "")
-      doITree(parser::parseString(""));
-    else {
-      if(outname.empty())
-	outname=(filename == "-") ? "out" : stripDir(basename);
-      doITree(parser::parseFile(filename));
-      outname="";
+  try {
+    if(getSetting<bool>("parseonly")) {
+      absyntax::file *tree = parser::parseFile(filename);
+      assert(tree);
+      em->sync();
+      if(!em->errors())
+	tree->prettyprint(cout, 0);
+      else status=false;
+    } else {
+      if(filename == "")
+	doITree(parser::parseString(""));
+      else {
+	if(getSetting<mem::string>("outname").empty())
+	  Setting("outname")=
+            (mem::string)((filename == "-") ? "out" : stripDir(basename));
+	doITree(parser::parseFile(filename));
+	Setting("outname")=(mem::string)"";
+      }
     }
+  } catch(handled_error) {
+    status=false;
   }
 }
 
 void doIPrompt() {
-  init();
-  outname="out";
+  cout << "Welcome to " << PROGRAM << " version " << VERSION
+       << " (to view the manual, type help)" << endl;
+  
+  interact::init_interactive();
+  
+  Setting("outname")=(mem::string)"out";
   
   iprompt i;
   do {
-    resetenv=false;
-    doICore(i);
-  } while(resetenv);
-  outname="";
+    try {
+      init();
+      virtualEOF=false;
+      i.set(parser::parseInteractive());
+      resetenv=false;
+      if(virtualEOF)
+	doICore(i);
+    } catch(handled_error) {
+      status=false;
+    }
+  } while(virtualEOF);
+  Setting("outname")=(mem::string)"";
+}
+
+// Run the config file.
+void doConfig(string filename) {
+  string file = settings::locateFile(filename);
+  if(!file.empty()) {
+    bool autoplain=getSetting<bool>("autoplain");
+    if(autoplain) Setting("autoplain")=false; // Turn off for speed.
+    doIFile(file);
+    if(autoplain) Setting("autoplain")=true;
+  }
 }
 
 } // namespace loop
@@ -304,21 +338,24 @@ void no_GCwarn(char *, GC_word) {}
 
 int main(int argc, char *argv[])
 {
-  setOptions(argc,argv);
-
 #ifdef USEGC
   GC_free_space_divisor = 2;
   GC_dont_expand = 0;
   GC_INIT();
-  if(!debug) GC_set_warn_proc(no_GCwarn);
+#endif  
+  
+  cout.precision(DBL_DIG);
+  setsignal(signalHandler);
+
+  setOptions(argc,argv);
+
+#ifdef USEGC
+  if(!getSetting<bool>("debug")) GC_set_warn_proc(no_GCwarn);
 #endif  
 
-  fpu_trap(trap);
-  setsignal(signalHandler);
+  fpu_trap(trap());
   if(interactive) signal(SIGINT,interruptHandler);
 
-  cout.precision(DBL_DIG);
-
   try {
     if(interactive)
       loop::doIPrompt();
