@@ -540,11 +540,16 @@ double path::directiontime(pair dir) const {
     if (t >= 0) return i+t;
     i++;
     if (cycles || i != n-1) {
-      pre = angle((point(i)-precontrol(i))*rot);
-      post = angle((postcontrol(i)-point(i))*rot);
-      if ((pre <= 0 && post >= 0 && pre >= post - PI) ||
-          (pre >= 0 && post <= 0 && pre <= post + PI))
-        return i;
+      pair Pre = (point(i)-precontrol(i))*rot;
+      pair Post = (postcontrol(i)-point(i))*rot;
+      static pair zero(0.0,0.0);
+      if(Pre != zero && Post != zero) {
+	pre = angle(Pre);
+	post = angle(Post);
+	if ((pre <= 0 && post >= 0 && pre >= post - PI) ||
+	    (pre >= 0 && post <= 0 && pre <= post + PI))
+	  return i;
+      }
     }
   }
   
@@ -554,10 +559,13 @@ double path::directiontime(pair dir) const {
 
 // {{{ Path Intersection Calculation
 
+static unsigned count;  
+unsigned maxIntersectCount=100000;
+
 // Algorithm derived from Knuth's MetaFont
 pair intersectcubics(solvedKnot left1, solvedKnot right1,
                      solvedKnot left2, solvedKnot right2,
-		     double fuzz, int depth=DBL_MANT_DIG)
+		     double fuzz, unsigned depth=DBL_MANT_DIG)
 {
   const pair F(-1,-1);
 
@@ -573,12 +581,14 @@ pair intersectcubics(solvedKnot left1, solvedKnot right1,
       box1.Max().gety()+fuzz >= box2.Min().gety() &&
       box2.Max().getx()+fuzz >= box1.Min().getx() &&
       box2.Max().gety()+fuzz >= box1.Min().gety()) {
-    if(lambda <= fuzz || depth == 0) return pair(0,0);
+    if(lambda <= fuzz || depth == 0 || count == 0)
+      return pair(0,0);
+    --depth;
+    --count;
     solvedKnot sn1[3], sn2[3];
     splitCubic(sn1,0.5,left1,right1);
     splitCubic(sn2,0.5,left2,right2);
     pair t;
-    depth--;
     if ((t=intersectcubics(sn1[0],sn1[1],sn2[0],sn2[1],fuzz,depth)) != F)
       return t*0.5;
     if ((t=intersectcubics(sn1[0],sn1[1],sn2[1],sn2[2],fuzz,depth)) != F)
@@ -609,6 +619,7 @@ pair intersectiontime(path p1, path p2, double fuzz=0.0)
     solvedKnot& left1=n1[i];
     solvedKnot& right1=(i == icycle) ? n1[0] : n1[i+1];
     for (int j = 0; j < L2; j++) {
+      count=maxIntersectCount;
       pair t=intersectcubics(left1,right1,
 			     n2[j],(j == jcycle) ? n2[0] : n2[j+1],fuzz);
       if (t != F) return t*0.5 + pair(i,j);
@@ -621,35 +632,38 @@ pair intersectiontime(path p1, path p2, double fuzz=0.0)
 ostream& operator<< (ostream& out, const path p)
 {
   size_t oldPrec = out.precision(6);
- 
+  
   int n = p.n;
   switch(n) {
   case 0:
     out << "<nullpath>";
-    return out;
+    break;
+    
   case 1:
     out << p.point(0);
-    return out;
+    break;
+
+  default:
+    out << p.point(0) << ".. controls " << p.postcontrol(0) << " and ";
+
+    for (int i = 1; i < n-1; i++) {
+      out << p.precontrol(i) << newl;
+
+      out << " .." << p.point(i);
+
+      out << ".. controls " << p.postcontrol(i) << " and ";
+    }
+    
+    out << p.precontrol(n-1) << newl
+	<< " .." << p.point(n-1);
+
+    if (p.cycles) 
+      out << ".. controls " << p.postcontrol(n-1) << " and "
+	  << p.precontrol(0) << newl
+	  << " ..cycle";
+    break;
   }
 
-  out << p.point(0) << ".. controls " << p.postcontrol(0) << " and ";
-
-  for (int i = 1; i < n-1; i++)
-  {
-    out << p.precontrol(i) << newl;
-
-    out << " .." << p.point(i);
-
-    out << ".. controls " << p.postcontrol(i) << " and ";
-  }
-  out << p.precontrol(n-1) << newl
-      << " .." << p.point(n-1);
-
-  if (p.cycles) 
-    out << ".. controls " << p.postcontrol(n-1) << " and "
-        << p.precontrol(0) << newl
-	<< " ..cycle";
-
   out.precision(oldPrec);
 
   return out;
