@@ -8,6 +8,7 @@
 #include "record.h"
 #include "inst.h"
 #include "runtime.h"
+#include "coder.h"
 
 namespace types {
 
@@ -41,4 +42,13 @@ trans::access *record::initializer() {
   return &a;
 }
 
+dummyRecord::dummyRecord(symbol *name) 
+  : record(name, new frame(0,0))
+{
+  // Encode the instructions to put an placeholder instance of the record on the
+  // stack.
+  trans::coder c(this, 0);
+  c.closeRecord();
+}
+
 } // namespace types
