@@ -55,19 +55,15 @@ const double ps2tex=1.0/tex2ps;
 template<class T>
 void texdocumentclass(T& out, bool pipe=false)
 {
-  if(pipe || !settings::texmode)
+  if(pipe || !settings::getSetting<bool>("inlinetex"))
     out << "\\documentclass[12pt]{article}" << newl;
 }
   
 template<class T>
 void texpreamble(T& out, stringlist& preamble=TeXpreamble)
 {
-  stringlist::iterator p=preamble.begin();
-  if(p != preamble.end()) {
-    TeXcontaminated=true;
-    for (; p != preamble.end(); ++p)
-      out << stripblanklines(*p);
-  }
+  for(stringlist::iterator p=preamble.begin(); p != preamble.end(); ++p)
+    out << stripblanklines(*p);
 }
   
 template<class T>
@@ -89,8 +85,9 @@ void texdefines(T& out, stringlist& preamble=TeXpreamble, bool pipe=false)
       << "\\def\\ASYscale(#1,#2)(#3,#4)(#5,#6)#7#8{%" << newl
       << "\\ASYalign(#1,#2)(#3,#4){#7}{\\scalebox{#5}[#6]{#8}}}%" << newl;
   
-  if(pipe || !settings::texmode)
+  if(pipe || !settings::getSetting<bool>("inlinetex"))
     out << "\\usepackage{pstricks,graphicx}" << newl;
+  if(pipe) out << "\\begin{document}" << newl;
 }
   
 class texfile : public gc {
