@@ -31,6 +31,7 @@ class varEntry;
 }
 namespace absyntax {
 class varinit;
+extern varinit *Default;
 }
 
 namespace types {
@@ -61,7 +62,7 @@ enum ty_kind {
 };
 
 // Forward declarations.
-struct ty;
+class ty;
 struct signature;
 
 // Checks if two types are equal in the sense of the language.
@@ -269,6 +270,7 @@ ty *primPen();
 ty *primPicture();
 ty *primFile();
 ty *primCode();
+ty *primArray();
   
 ty *boolArray();
 ty *intArray();
@@ -300,13 +302,21 @@ struct formal {
   ty *t;
   symbol *name;
   absyntax::varinit *defval;
-  bool xplicit; // since explicit is a C++ keyword
+  bool Explicit;
   
   formal(ty *t,
          symbol *name=0,
-         absyntax::varinit *defval=0,
-         bool xplicit=false)
-    : t(t), name(name), defval(defval), xplicit(xplicit) {}
+         bool optional=false,
+         bool Explicit=false)
+    : t(t), name(name),
+      defval(optional ? absyntax::Default : 0), Explicit(Explicit) {}
+
+  formal(ty *t,
+         const char *name,
+         bool optional=false,
+         bool Explicit=false)
+    : t(t), name(symbol::trans(name)),
+      defval(optional ? absyntax::Default : 0), Explicit(Explicit) {}
 
   friend ostream& operator<< (ostream& out, const formal& f);
 };
@@ -335,7 +345,7 @@ struct signature : public gc {
   }
 
   void addRest(formal f) {
-    rest = f;
+    rest=f;
   }
 
   bool hasRest() {
@@ -409,7 +419,7 @@ struct function : public ty {
   }
 
   void print(ostream& out) const
-    { out << *result << ' ' << sig; }
+    { out << *result << sig; }
 
   void printVar (ostream& out, symbol *name) const {
     result->printVar(out,name);
