@@ -24,6 +24,12 @@
 
 using namespace settings;
 
+#ifdef __CYGWIN__
+#define CERR cout
+#else
+#define CERR cerr
+#endif  
+
 bool False=false;
 
 string stripext(const string& name, const string& ext)
@@ -77,7 +83,7 @@ string buildname(string name, string suffix, string aux, bool stripdir)
 {
   if(stripdir) stripDir(name);
     
-  name = stripext(name,outformat);
+  name = stripext(name,getSetting<mem::string>("outformat"));
   name += aux;
   if(!suffix.empty()) name += "."+suffix;
   return name;
@@ -104,7 +110,7 @@ char **args(const char *command)
 {
   if(command == NULL) return NULL;
   
-  int n=0;
+  size_t n=0;
   char **argv=NULL;  
   for(int pass=0; pass < 2; ++pass) {
     if(pass) argv=new char*[n+1];
@@ -142,10 +148,20 @@ char **args(const char *command)
 
 void execError(const char *command, const char *hint, const char *application)
 {
-    cerr << "Cannot execute " << command << endl;
+    CERR << "Cannot execute " << command << endl;
+    if(application == "") application=hint;
+    string s=string(hint);
+    transform(s.begin(), s.end(), s.begin(), toupper);        
     if(hint) 
-      cerr << "Please set the environment variable " << hint << endl
-	   << "to the location of " << application << endl;
+      CERR << "Please put in " << getSetting<mem::string>("config")
+	   << ": " << endl << endl
+	   << "import settings;" << endl
+           << hint << "=\"PATH\";" << endl << endl
+	   << "where PATH denotes the correct path to " 
+	   << application << "." << endl << endl
+	   << "Alternatively, set the environment variable ASYMPTOTE_" << s 
+	   << endl << "or use the command line option -" << hint 
+	   << "=\"PATH\"" << endl;
     exit(-1);
 }
 						    
@@ -155,20 +171,23 @@ int System(const char *command, bool quiet, bool wait,
   int status;
 
   if(!command) return 1;
-  if(verbose > 1) cerr << command << endl;
+  if(settings::verbose > 1) cerr << command << endl;
 
   cout.flush(); // Flush stdout to avoid duplicate output.
     
+  char **argv=args(command);
+
   int pid = fork();
   if(pid == -1)
     camp::reportError("Cannot fork process");
   
-  char **argv=args(command);
   if(pid == 0) {
     if(interact::interactive) signal(SIGINT,SIG_IGN);
     if(quiet) close(STDOUT_FILENO);
-    if(argv) execvp(argv[0],argv);
-    execError(argv[0],hint,application);
+    if(argv) {
+      execvp(argv[0],argv);
+      execError(argv[0],hint,application);
+    }
   }
 
   if(ppid) *ppid=pid;
@@ -177,7 +196,7 @@ int System(const char *command, bool quiet, bool wait,
       if(errno == ECHILD) return 0;
       if(errno != EINTR) {
         ostringstream msg;
-        msg << "Command " << command << " failed";
+        msg << "Command failed: " << command;
         camp::reportError(msg);
       }
     } else {
@@ -193,7 +212,7 @@ int System(const char *command, bool quiet, bool wait,
 	return WEXITSTATUS(status);
       } else {
         ostringstream msg;
-        msg << "Command " << command << " exited abnormally";
+        msg << "Command exited abnormally: " << command;
         camp::reportError(msg);
       }
     }
